#ifndef _GENERATED_CPPUTESTGENERATEDCONFIG_H
#define _GENERATED_CPPUTESTGENERATEDCONFIG_H 1
 
/* generated/CppUTestGeneratedConfig.h. Generated automatically at end of configure. */
/* config.h.  Generated from config.h.in by configure.  */
/* config.h.in.  Generated from configure.ac by autoheader.  */

/* Define to 1 if you have the <dlfcn.h> header file. */
#ifndef CPPUTEST_HAVE_DLFCN_H
#define CPPUTEST_HAVE_DLFCN_H 1
#endif

/* Define to 1 if you have the `fork' function. */
#ifndef CPPUTEST_HAVE_FORK
#define CPPUTEST_HAVE_FORK 1
#endif

/* Define to 1 if you have the `gettimeofday' function. */
#ifndef CPPUTEST_HAVE_GETTIMEOFDAY
#define CPPUTEST_HAVE_GETTIMEOFDAY 1
#endif

/* Define to 1 if you have the <inttypes.h> header file. */
#ifndef CPPUTEST_HAVE_INTTYPES_H
#define CPPUTEST_HAVE_INTTYPES_H 1
#endif

/* Define to 1 if the system has the type `long long int'. */
#ifndef CPPUTEST_HAVE_LONG_LONG_INT
#define CPPUTEST_HAVE_LONG_LONG_INT 1
#endif

/* Define to 1 if you have the <memory.h> header file. */
#ifndef CPPUTEST_HAVE_MEMORY_H
#define CPPUTEST_HAVE_MEMORY_H 1
#endif

/* Define to 1 if you have the `memset' function. */
#ifndef CPPUTEST_HAVE_MEMSET
#define CPPUTEST_HAVE_MEMSET 1
#endif

/* Define if you have POSIX threads libraries and header files. */
/* #undef HAVE_PTHREAD */

/* Define to 1 if you have the `pthread_mutex_lock' function. */
#ifndef CPPUTEST_HAVE_PTHREAD_MUTEX_LOCK
#define CPPUTEST_HAVE_PTHREAD_MUTEX_LOCK 1
#endif

/* Define to 1 if you have the <stddef.h> header file. */
#ifndef CPPUTEST_HAVE_STDDEF_H
#define CPPUTEST_HAVE_STDDEF_H 1
#endif

/* Define to 1 if you have the <stdint.h> header file. */
#ifndef CPPUTEST_HAVE_STDINT_H
#define CPPUTEST_HAVE_STDINT_H 1
#endif

/* Define to 1 if you have the <stdlib.h> header file. */
#ifndef CPPUTEST_HAVE_STDLIB_H
#define CPPUTEST_HAVE_STDLIB_H 1
#endif

/* Define to 1 if you have the `strdup' function. */
#ifndef CPPUTEST_HAVE_STRDUP
#define CPPUTEST_HAVE_STRDUP 1
#endif

/* Define to 1 if you have the <strings.h> header file. */
#ifndef CPPUTEST_HAVE_STRINGS_H
#define CPPUTEST_HAVE_STRINGS_H 1
#endif

/* Define to 1 if you have the <string.h> header file. */
#ifndef CPPUTEST_HAVE_STRING_H
#define CPPUTEST_HAVE_STRING_H 1
#endif

/* Define to 1 if you have the `strstr' function. */
#ifndef CPPUTEST_HAVE_STRSTR
#define CPPUTEST_HAVE_STRSTR 1
#endif

/* Define to 1 if you have the <sys/stat.h> header file. */
#ifndef CPPUTEST_HAVE_SYS_STAT_H
#define CPPUTEST_HAVE_SYS_STAT_H 1
#endif

/* Define to 1 if you have the <sys/time.h> header file. */
#ifndef CPPUTEST_HAVE_SYS_TIME_H
#define CPPUTEST_HAVE_SYS_TIME_H 1
#endif

/* Define to 1 if you have the <sys/types.h> header file. */
#ifndef CPPUTEST_HAVE_SYS_TYPES_H
#define CPPUTEST_HAVE_SYS_TYPES_H 1
#endif

/* Define to 1 if you have the <unistd.h> header file. */
#ifndef CPPUTEST_HAVE_UNISTD_H
#define CPPUTEST_HAVE_UNISTD_H 1
#endif

/* Define to 1 if the system has the type `unsigned long long int'. */
#ifndef CPPUTEST_HAVE_UNSIGNED_LONG_LONG_INT
#define CPPUTEST_HAVE_UNSIGNED_LONG_LONG_INT 1
#endif

/* Define to 1 if you have the `vfork' function. */
#ifndef CPPUTEST_HAVE_VFORK
#define CPPUTEST_HAVE_VFORK 1
#endif

/* Define to 1 if you have the <vfork.h> header file. */
/* #undef HAVE_VFORK_H */

/* Define to 1 if `fork' works. */
#ifndef CPPUTEST_HAVE_WORKING_FORK
#define CPPUTEST_HAVE_WORKING_FORK 1
#endif

/* Define to 1 if `vfork' works. */
#ifndef CPPUTEST_HAVE_WORKING_VFORK
#define CPPUTEST_HAVE_WORKING_VFORK 1
#endif

/* Include the GTest-related tests in the build */
/* #undef INCLUDE_GTEST_TESTS */

/* disable long long */
/* #undef LONG_LONG_DISABLED */

/* Define to the sub-directory where libtool stores uninstalled libraries. */
#ifndef CPPUTEST_LT_OBJDIR
#define CPPUTEST_LT_OBJDIR ".libs/"
#endif

/* memory leak detection disabled */
/* #undef MEM_LEAK_DETECTION_DISABLED */

/* Name of package */
#ifndef CPPUTEST_PACKAGE
#define CPPUTEST_PACKAGE "cpputest"
#endif

/* Define to the address where bug reports for this package should be sent. */
#ifndef CPPUTEST_PACKAGE_BUGREPORT
#define CPPUTEST_PACKAGE_BUGREPORT "https://github.com/cpputest/cpputest"
#endif

/* Define to the full name of this package. */
#ifndef CPPUTEST_PACKAGE_NAME
#define CPPUTEST_PACKAGE_NAME "CppUTest"
#endif

/* Define to the full name and version of this package. */
#ifndef CPPUTEST_PACKAGE_STRING
#define CPPUTEST_PACKAGE_STRING "CppUTest 4.0"
#endif

/* Define to the one symbol short name of this package. */
#ifndef CPPUTEST_PACKAGE_TARNAME
#define CPPUTEST_PACKAGE_TARNAME "cpputest"
#endif

/* Define to the home page for this package. */
#ifndef CPPUTEST_PACKAGE_URL
#define CPPUTEST_PACKAGE_URL ""
#endif

/* Define to the version of this package. */
#ifndef CPPUTEST_PACKAGE_VERSION
#define CPPUTEST_PACKAGE_VERSION "4.0"
#endif

/* Define to necessary symbol if this constant uses a non-standard name on
   your system. */
/* #undef PTHREAD_CREATE_JOINABLE */

/* Define to 1 if you have the ANSI C header files. */
#ifndef CPPUTEST_STDC_HEADERS
#define CPPUTEST_STDC_HEADERS 1
#endif

/* Standard C++ library disabled */
/* #undef STD_CPP_LIB_DISABLED */

/* Standard C library disabled */
/* #undef STD_C_LIB_DISABLED */

/* Version number of package */
#ifndef CPPUTEST_VERSION
#define CPPUTEST_VERSION "4.0"
#endif

/* Define for Solaris 2.5.1 so the uint32_t typedef from <sys/synch.h>,
   <pthread.h>, or <semaphore.h> is not used. If the typedef were allowed, the
   #define below would cause a syntax error. */
/* #undef _UINT32_T */

/* Define for Solaris 2.5.1 so the uint64_t typedef from <sys/synch.h>,
   <pthread.h>, or <semaphore.h> is not used. If the typedef were allowed, the
   #define below would cause a syntax error. */
/* #undef _UINT64_T */

/* Define for Solaris 2.5.1 so the uint8_t typedef from <sys/synch.h>,
   <pthread.h>, or <semaphore.h> is not used. If the typedef were allowed, the
   #define below would cause a syntax error. */
/* #undef _UINT8_T */

/* Define to `__inline__' or `__inline' if that's what the C compiler
   calls it, or to nothing if 'inline' is not supported under any name.  */
#ifndef __cplusplus
/* #undef inline */
#endif

/* Define to the type of a signed integer type of width exactly 16 bits if
   such a type exists and the standard includes do not define it. */
/* #undef int16_t */

/* Define to the type of a signed integer type of width exactly 32 bits if
   such a type exists and the standard includes do not define it. */
/* #undef int32_t */

/* Define to the type of a signed integer type of width exactly 64 bits if
   such a type exists and the standard includes do not define it. */
/* #undef int64_t */

/* Define to the type of a signed integer type of width exactly 8 bits if such
   a type exists and the standard includes do not define it. */
/* #undef int8_t */

/* Define to `int' if <sys/types.h> does not define. */
/* #undef pid_t */

/* Define to `unsigned int' if <sys/types.h> does not define. */
/* #undef size_t */

/* Define to the type of an unsigned integer type of width exactly 16 bits if
   such a type exists and the standard includes do not define it. */
/* #undef uint16_t */

/* Define to the type of an unsigned integer type of width exactly 32 bits if
   such a type exists and the standard includes do not define it. */
/* #undef uint32_t */

/* Define to the type of an unsigned integer type of width exactly 64 bits if
   such a type exists and the standard includes do not define it. */
/* #undef uint64_t */

/* Define to the type of an unsigned integer type of width exactly 8 bits if
   such a type exists and the standard includes do not define it. */
/* #undef uint8_t */

/* Define as `fork' if `vfork' does not work. */
/* #undef vfork */
 
/* once: _GENERATED_CPPUTESTGENERATEDCONFIG_H */
#endif
